############################################################
#                                                          #
#              Information                                 #
#                                                          #
############################################################

# Dates: 1/01/2021 till 30/12/2021 We dont have data for 31 Dec. and the code don't resist it. We need more robustness   
# Time zone and hour format: Hour of the day is shown in UTC. Value at Hour 1 correspond to the hour starting at 1 and ending at 2. As April falls in summer time (UTC + 2 hours), to convert from local time to UTC we move two hours backwards. So, 7 am local time is 5 am UTC.   
# Pollutant: Depends on the variable pollutant in config_file   
# Catalunya    
# Sites: monitoring stations, the number of them depends on seet_stations code
# Observations source: 
# Models intercompared:  
# Raw model from Caliope
# LOOCV applied to Caliope
#-------------------------------------------------------------------------------
# Measure the time it takes to run the analysis
start_time <- Sys.time()
#-------------------------------------------------------------------------------
#              LOOCV TO CALIOPE DATA WITH CATALONIA OBSERVATIONS
#-------------------------------------------------------------------------------

############################################################
#                                                          #
#              Set libraries                               #
#                                                          #
############################################################
# ! check the version of the package or R if any library or function does not work correctly

library(urbanR)
library(RNetCDF)
library(sp)
library(sf)
#library(mapview)
library(ncdf4)
library(rgeos) 
library(ggplot2)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr 
library(dplyr)
library(broom)
library(magrittr)
library(ggpubr)
library(raster)
library(rgdal)
#library(tmap)
library(gstat)
library(sp)
library(abind)
library(openair)
#library(Metrics)

############################################################
#                                                          #
#              LOAD config_file                            #
#                                                          #
############################################################

args = commandArgs(TRUE)
if (length(args) > 0) {
  configFile_path <- args[1]
} else {
  cat("\n***WARNING: no ConfigFile was provided. Using default ConfigFile.\n")
}

#config_file it's a file where we change the data, where we want to save the results, etc. so now we extract these data from it:
#configFile_path <- "/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/config_file_2021.yml"

config          <- yaml::yaml.load_file(configFile_path)
sim_folder_path <- config$sim_folder_path
obs_path        <- config$obs_path
model_path      <- config$model_path
save_result     <- config$save_result
pollutant       <- config$pollutant 
print(paste0("Selected pollutant: ", pollutant))
# If data_resolution is 1, we will read the model and the observations hourly. if 2, we will read it daily
data_resolution <- config$data_resolution #em aquest fitxer no ho he implementat, en el UK_+_LOOCV.R YES

date_time_end   <- config$date_time_end
date_time_begin <- config$date_time_begin

name            <- config$doc_name
variogram_path  <- config$variogram_path
path_code       <- config$path_code

set_stations       <- config$set_stations
set_observations   <- config$set_observations 
set_model          <- config$set_model
execute_loocv      <- config$execute_loocv
execute_statistics <- config$execute_statistics

save_model                 <- config$save_model
save_obs                   <- config$save_obs
save_OK                    <- config$save_OK
save_LOOCV                 <- config$save_LOOCV
save_stations              <- config$save_stations

save_general_statistics    <- config$save_general_statistics #Images
save_time_series           <- config$save_time_series #Images 1
save_temporal_variation    <- config$save_temporal_variation #Imatges 2

#obs_path <-"/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/observacions/obs_o3/"
#obs_path <-"/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/observacions/obs_pm10/"
# As we had problems with Ordinary and Universal Kriging variograms, we saved one of each to read them in case we have problems with the one executed for a specific date or hour
variogram_ok <- paste0(variogram_path, "Variogram_OK.rds")
variogram_uk <- paste0(variogram_path, "Variogram_UK.rds")

############################################################
#                                                          #
#              Set Units                                   #
#                                                          #
############################################################
print("Setting units of obs and model")
# We need to adjust the units of the observations to ug/m3, so first we pick up the constant depending on the pollutant that we are working with
pol          <- c("no2","o3","pm10","pm25")
k            <- c(1.881, 1.963, 1, 1) # Based on obs constant
# We will use the limit parameter to avoid observations that we consider wrong in loocv loop. As for NO2 the alert is established at 400 ug/m3, and 240 for O3 we will put this number as the boundary
limit        <- c(400, 240, 100, 100) #the limit for pm10 is 50 ug/m3, 25 ug/m3, but i put 100 just in case 

position     <- match(pollutant, pol) 
number       <- match(pol[position], pol)

limit        <- limit[number]
obs_constant <- k[number]

#pol <- c("O3", "NO2","SO2","NO","CO","NH3","HNO3","HONO","PM10","PM25","wspd10","wdir10","PRES","WD","WS","PM10SO4","PM10NO3","PM10NH4","PM10NA","PM10CL","PM10BC","PM10SOA","PM10PPM","PM10DU","PM10POA","PM10UNSPEC", "EC")
# this is not correct, EC is provided by reduce in kgm3, thats why we multiply by 1e09 to obtain ugm3. hence it is not a ppm to ugm3 conversion
#k   <- c(1962, 1881 ,2620 ,1227,1145,695  ,2576 ,1922 ,1e09 ,1e09 ,1 ,1 ,0.01 ,1 ,1 ,1e09 ,1e09 ,1e09 ,1e09 ,1e09 ,1e09 ,1e09 ,1

#to arrange set_stations or observations, we need to configure name_pollutant, based in how the script name with the data is saved
if(pollutant %in% c("no2","o3")){
  name_pollutant   <- paste0("sconc",pollutant)
  mod_constat      <- k[number]*1e03
  min_obs          <- 20 #min nº of obs that we consider to apply uk and loocv
  
} else{
  #at the moment pm10 and pm2.5
  name_pollutant  <- pollutant
  mod_constat     <- k[number]*1e09 #in the script of pm the units are kg/m3 but we are sure that they are worng, and they are in yg/m3
  min_obs         <- 5 #min nº of obs that we consider to apply uk and loocv
}

############################################################
#                                                          #
#              Set temporal context                        #
#                                                          #
############################################################
print("setting dates")
#date_times et fa un super vector d'una  columna, dia per dia, hora per hora des de l'inici fins a la data final p.ex: 2021-01-01 00:00:00
date_times <- data_frame_dates(date_time_begin = date_time_begin,
                               date_time_end   = date_time_end)

#dates: Llista de caracters anymesdia p.ex: 20210101
dates <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%m", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%d", tz = "UTC"))))
#year_month: igual que date pero anymes p.ex: 202101
year_month <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                            as.character(strftime(date_times[, 1], format="%m", tz = "UTC"))))

############################################################
#                                                          #
#                 Load Local Stations                      #
#                                                          #
############################################################
print("Load Local Stations")
# local_stations: special points data frame with two cols. For example, for NO2 pollutant [NO2 and ref_obs-reference of station-], at the moment we have 72 stations in Cat for this pollutant 

if (set_stations){
  source(paste0(path_code,"set_stations.R"))
  print("Set stations load: done")
} else {
  local_stations <- readRDS(file = paste0(save_result,"local_stations_",pollutant,".rds"))
  print("Read local stations: done")
}


############################################################
#                                                          #
#                 Load Observations                        #
#                                                          #
############################################################
#we have a doc where we have the reference of the stations that are in catalonia, if we want to change or amplify them we can change the coordinates of the box in set_stations script

print("Load observations")

# we prepare a matrix where we will save the observations:
obs                  <- matrix(NA,ncol=(nrow(local_stations@data)),nrow=0)
df_obs_all           <- data.frame(obs)
dates_all            <- data.frame(matrix(NA,ncol= 1,nrow=0))
#we transform from a matrix to a data frame, with the reference of the station as a name of the col
colnames(df_obs_all) <- c(local_stations@data$ref_obs)
colnames(dates_all)  <- c("date")

# finally we extract the observations of certain year_month
for (y_month in year_month) {
  
  netcdf_obs    <- RNetCDF::open.nc(paste0(obs_path, name_pollutant, "_", y_month, ".nc"))

  code_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'station_reference')
  obs_all       <- RNetCDF::var.get.nc(netcdf_obs, name_pollutant)
  
  year            <- as.numeric(substr(y_month, 1, 4))
  month           <- as.numeric(substr(y_month, 5, 6))
  num_days        <- length(RNetCDF::var.get.nc(netcdf_obs, "time"))/24
  date_times_loc  <- data_frame_dates(c(year,month,01,00) , c(year,month,num_days,23)) # p.ex: 2021-01-18 01:00:00
	dates_all       <- rbind(dates_all,date_times_loc)

  pos_local_stations  <- which(code_obs%in%local_stations@data$ref_obs)
  cat_obs             <- obs_all[,pos_local_stations]
  cat_ref             <- code_obs[pos_local_stations]
  
  
  obs_month               <- matrix(NA,ncol=length(cat_obs[1,]),nrow=length(cat_obs[,1])) 
  #cat_obs matriu de 72 col (estacions) i en aquest cas p.ex: 744 (31 dies de generx24h) i fem una obs month buida
  df_obs_month            <- data.frame(obs_month)
  colnames(df_obs_month)  <- paste0(cat_ref)
  df_obs_month[,]         <- cat_obs*obs_constant
  
  df_obs_all              <- rbind.fill(df_obs_all, df_obs_month) #in case there is no data on a certain date, it fills it with NA (rbind.fill)
  
    if ( ncol(df_obs_all)!=ncol(df_obs_month) ) {
      print(paste0("ERROR: number of monitoring stations is not constant when loading observations. ","Filled with NA in: ",y_month))
      print(paste0("This month it has: ", ncol(df_obs_month), " from: ", ncol(df_obs_all), " setted initially" ))
    }
}

df_obs_all <- cbind(dates_all,df_obs_all)

#time_subset permits us to select the observations at the same dates as the model (remember: at first we extract observations monthly)
df_obs_sub <- time_subset(data = df_obs_all, time_var_name = "date",
                          date_time_begin = date_time_begin,
                          date_time_end = date_time_end)
if (save_obs){
  saveRDS(object=df_obs_all, file=paste0(save_result,"df_obs_",pollutant,".rds"))
  print("Observations saved")
} else {
  print("Observations have not been saved")
}

############################################################
#                                                          #
#                 Load Model                               #
#                                                          #
############################################################
print("Load Model")

n_lat                <- 278        #latitude
n_lon                <- 298        #longitude
model                <- array(dim=c(n_lat,n_lon,0)) #we prepare an array to put the date in

# We download the model by days
for (date in dates) {
  
  #date<-"20210101"
  conc_path_day  <- paste0(date, "00")
  file           <- paste0(name_pollutant, "_", conc_path_day, ".nc", sep="")
  file_path      <- paste0(model_path, file)
  
  if (file.exists(file_path)) {
    netcdf_model   <- RNetCDF::open.nc(file_path) #Dintre de la llibreria executem la funció open i aplica-ho al fitxer obs
    
    #Extraiem les dades del model
    model_day      <- RNetCDF::var.get.nc(netcdf_model,name_pollutant) #Concentracions del NO2
    model_day      <- model_day[,,1:24]
    model_day      <- model_day*mod_constat #les dades del model les tens en ppm i nosaltres ho volem en microg/m3
  }else{
    model_day       <- array(NaN, dim=c(n_lat,n_lon,24)) #if we dont have data for a certain day its filled by NaN
  }
  model            <- abind(model,model_day,along=3)
}


lat_model      <- RNetCDF::var.get.nc(netcdf_model, 'lat') #Latitut
lat_model      <- sapply(lat_model, as.numeric)            #era una large matrix, ara es una large numeric
lon_model      <- RNetCDF::var.get.nc(netcdf_model, 'lon') #Longitud
lon_model      <- sapply(lon_model, as.numeric)

if (save_model){
  saveRDS(object=model, file=paste0(save_result,"model_",pollutant,".rds"))
  print("Model saved")
} else {
  print("Model have not been saved")
}  

############################################################
#                                                          #
#                 Loop Universal Kriging                   #
#                                                          #
############################################################
print("Loop Universal Kriging")
compute_in_UTM        <- TRUE #We want to transform the coordinates from CSR to UTM, it is faster in Universal Kriging
crs_model_utm         <- "+proj=utm +zone=31 +ellps=intl +units=m +no_defs"
crs_model             <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0" #coordinates sistem

#LOOCV_total and llista_uk_total is created empty to fill it later with LOOCV and raw model data respectivelly
LOOCV_total           <- matrix(NA,ncol=nrow(local_stations@data),nrow=0) #list(length=nrow(date_times)) #we will have a large list with a data.frame for each hour
colnames(LOOCV_total) <- local_stations$ref_obs #we want a matrix with empty data and the name of stations

llista_uk_total           <- matrix(NA,ncol=nrow(local_stations@data),nrow=0) #list(length=nrow(date_times)) #we will have a large list with a data.frame for each hour
colnames(llista_uk_total) <- local_stations$ref_obs #we want a matrix with empty data and the name of stations

#starting OK and LOOCV:
for (hour in 1:nrow(date_times)) {
  print(paste0("loop UK number:",hour))
  df_model <- model[,,hour]
  
  #if all the model for a certain hour is nan we cannot apply UK
  if ( sum(is.nan(df_model))!=length(df_model)  ){
    # if we have less than 20 obs:
    if(!(sum(is.na(df_obs_sub[hour,-1]))<min_obs)){
      print(paste0("For hour : ", hour, " we don't have the minimum obs to apply UK"))
    }
    
    #To aply UK we need a minimum of observations, in this case if we have less than 20 observations for a certain hour we will fill it with NA
    if(sum(is.na(df_obs_sub[hour,-1]))<min_obs) {
      
      #print(df_obs_all[hour,1])
      df_obs   <- df_obs_sub[hour,-1]
      #sum(is.na(df_obs)). Hem de marcar-nos fins a quantes observacions fem l'UK, i en cas de no ternir-ne suficients si agafar un variograma del UK previ o bé saltar a l'else
      
      # 1. Interpolate model values at obs locations
      #    1.1. Set spatialpointsdataframe
      #raw model:
      df_model                  <- data.frame(lat=lat_model,lon=lon_model,pollutant=sapply(df_model,as.matrix))
      coordinates(df_model)     <- ~lon+lat
      sp::proj4string(df_model) <- crs_model
      
      #config the coordinates in UTM
      if (compute_in_UTM) {
        df_model <- spTransform(df_model, sp::CRS(crs_model_utm))
      }
      
      #observations:
      df_obs                     <- data.frame(lat=local_stations@coords[,2],lon=local_stations@coords[,1],pollutant=as.numeric(df_obs[1,]),code=colnames(df_obs))
      df_obs                     <- na.omit(df_obs)
      df_obs[df_obs$pollutant<0,"pollutant"] <- NA
      df_obs                     <- na.omit(df_obs)
      
      #To avoid what we suppose wrong observations, we will subtitute the values higher than limit microg/m3
      w <- length(which(df_obs$pollutant>limit))
      if(w>=1){
        df_obs[df_obs$pollutant>limit,"pollutant"] <- limit
        print(paste0("The observations has been modified to exceed ", limit, " límit value. The number of observations modified are : ", w, " in the hour:", hour))
      }
      
      coordinates(df_obs)        <- ~lon+lat
      sp::proj4string(df_obs)    <- crs_model
      
      #config the coordinates in UTM
      if (compute_in_UTM) {
        df_obs <- spTransform(df_obs, sp::CRS(crs_model_utm))
      }
      
      
      #    1.2 Compute Variogram
      set.seed(1) #we are picking random points in the next line, set.seed permits us to have same points in each loop
      pick_random_points  <- sample(1:nrow(df_model),2000)
      pollutant.fit_ok          <- automap::autofitVariogram(formula=pollutant~1, input_data=df_model[pick_random_points,],model="Ste")
      # Hem posat model='Ste' pk la segona hora del 01/06/2021 peta el variograma, mirar bé perque no s'ajusa
      
      #    1.3 Compute ordinary Kriging
      
      #Del model interpolo cap a les coordenades de les obs 'OK'
      ok_gstat <- gstat(formula=pollutant~1, data = df_model, model = pollutant.fit_ok$var_model, nmax=100)
      
      #Interpolacio del model a les coordenades del conjunt
      OK       <- predict(object=ok_gstat, newdata=df_obs)
      
      #If the variogram has singular matrix then it produces NAN's, so in that case we take a good Variogram instead:
      if(any(is.na(OK@data$var1.pred))){
        print("WARNING: VARIOGRAM DIVERGED! Using a precalculated VARIOGRAM")
        pollutant.fit_ok=readRDS(variogram_ok)
        ok_gstat <- gstat(formula=pollutant~1, data = df_model, model = pollutant.fit_ok$var_model, nmax=100)
        
        #Interpolacio del model a les coordenades del conjunt
        OK <-predict(object=ok_gstat, newdata=df_obs)
      }
      
      # Arrange data obs-raw_model
      llista_uk                      <- data.frame(df_obs@coords)
      llista_uk$pollutant_obs        <- df_obs@data$pollutant
      llista_uk$pollutant_model      <- OK$var1.pred
      coordinates(llista_uk)         <- ~lon+lat
      
      #config the coordinates in UTM
      if (compute_in_UTM) {
        sp::proj4string(llista_uk) <- crs_model_utm
      } else {
        sp::proj4string(llista_uk) <- crs_model
      }
      
      # Compute UK variogram
      
      # in exceptional cases we have problems with autofitVariogram (eg: date="20211108" at 18 h) we still don't know why (maybe few points to execute the variogram, also we see negative range...)
      
      pollutant.fit_uk <- automap::autofitVariogram(formula=pollutant_obs~pollutant_model,input_data=llista_uk, verbose=FALSE)
      
      #Emergency Variogram if range of the current Variogram it's negative
      if(any(pollutant.fit_uk$var_model$range<0)){
        print('UK Variogram had negative range')
        pollutant.fit_uk=readRDS(variogram_uk)
      }
      
      #coef(lm(pollutant_obs ~  pollutant_model , llista_uk))['pollutant_model'] is the same as slope:
      slope <- summary(lm(pollutant_obs ~  pollutant_model , llista_uk))$coefficients[,1][-1] 
      #p     <-summary(lm(pollutant_obs ~  pollutant_model , llista_uk))$coefficients[,4][-1] #p value
      if(base::isTRUE(slope[1] < 0)){
        
        ok_loocv              <- NULL
        ok_loocv              <- as.data.frame(llista_uk$pollutant_model)
        colnames(ok_loocv)    <- c("var1.pred")
        coordinates(ok_loocv) <- llista_uk@coords
        
        #config the coordinates in UTM
        if (compute_in_UTM) {
          sp::proj4string(ok_loocv) <- crs_model_utm
        } else {
          sp::proj4string(ok_loocv) <- crs_model
        }
        
        print(paste0("The slope in hour ", hour, " is ", slope, ". We did not apply LOOCV, raw-model value is picked for this hour"))
        
      }
      
      else {
        
        # LOOCV
        ok_loocv             <- krige.cv(pollutant_obs~pollutant_model,llista_uk,pollutant.fit_uk$var_model)
        
      }
      
      # To check everything is fine, compute UK using all stations
      #uk_gstat<- gstat(formula=NO2_obs~NO2_model,data=llista_uk, model = NO2.fit_uk$var_model)
      #ok_loocv <-predict(object=uk_gstat, newdata=llista_uk)
      
      
      # dtaframe amb columnes df_obs$code ajuntar LOOCV based on code
      
      # Save LOOCV and raw_model
      uk_loocv_h           <- matrix(data=ok_loocv@data$var1.pred, nrow=1)
      colnames(uk_loocv_h) <- df_obs$code
      
      llista_uk_h           <- matrix(data=llista_uk@data$pollutant_model, nrow=1)
      colnames(llista_uk_h) <- df_obs$code
      
      llista_uk_total <-  plyr::rbind.fill(as.data.frame(llista_uk_total), as.data.frame(llista_uk_h))
      LOOCV_total     <-  plyr::rbind.fill(as.data.frame(LOOCV_total),as.data.frame(uk_loocv_h))
      
    }
    
  }else{
    uk_loocv_h            <- matrix(data=NA, nrow=1,ncol=(length(df_obs_all)-1))
    colnames(uk_loocv_h)  <- colnames(df_obs_all[,-1])
    LOOCV_total           <- plyr::rbind.fill(LOOCV_total,as.data.frame(uk_loocv_h))
    
    llista_uk_h           <- matrix(data=NA, nrow=1,ncol=(length(df_obs_all)-1))
    colnames(llista_uk_h) <- colnames(df_obs_all[,-1])
    llista_uk_total       <- plyr::rbind.fill(llista_uk_total,as.data.frame(llista_uk_h))
    
  }
}

LOOCV_total_df       <- as.data.frame(LOOCV_total)
LOOCV_total_df$dates <- date_times$date

llista_uk_total_df       <- as.data.frame(llista_uk_total)
llista_uk_total_df$dates <- date_times$date

#Arrange LOOCV_total_df to remove negative values:

LOOCV_total_df_wd        <- LOOCV_total_df %>% dplyr::select(-dates)
# Realment es podria fer abans de juntar el date al sortir del bucle. Ho deixo així perquè tenim LOOCV guardat amb la data
dates                    <- LOOCV_total_df %>% dplyr::select(dates) 

for(i in 1:nrow(local_stations)){
  LOOCV_total_df_wd[[i]] <- replace(LOOCV_total_df[[i]], LOOCV_total_df[[i]]<0, 0)
}

LOOCV_total_df <- cbind(LOOCV_total_df_wd,dates)

#Verification of negative values 'which(llista_uk_total_df<0)'. At the moment we don't apply it to llista_uk_total because it doesn't have negative values

if (save_OK){
  saveRDS(object=llista_uk_total_df, file=paste0(save_result,"llista_uk_",pollutant,"_df.rds"))
  print("Llista OK saved")
} else {
  print("Llista OK have not been saved")
}

if (save_LOOCV){
  saveRDS(object=LOOCV_total_df, file=paste0(save_result,"LOOCV_",pollutant,"_df.rds"))
  print("LOOCV saved")
} else {
  print("LOOCV have not been saved")
}

############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################
print("Evaluating statistics")

# it alerts us if the dates in model do not coincide with the dates of the obs
if ( any(date_times$date != df_obs_sub$date))  {
  print("ERROR dates from model and observations are wrong")
}

#-------------------------------------------------------------------------------
# We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Comparation using all stations concatenated:
raw_model_total       <- NULL
corr_model_total      <- NULL 

for (i in 1:nrow(local_stations)) {

  raw_model_estacio <- data.frame(obs = df_obs_sub[ ,colnames(llista_uk_total)[i]],
                           mod = llista_uk_total_df[ ,colnames(llista_uk_total)[i]],
                           model="raw_model")
  
  corr_model_estacio <- data.frame(obs = df_obs_sub[ ,colnames(llista_uk_total)[i] ],
                          mod = LOOCV_total_df[ ,colnames(llista_uk_total)[i]],
                          model="corr_model")
  #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
  raw_model_total   <- rbind(raw_model_total,raw_model_estacio) 
  corr_model_total  <- rbind(corr_model_total,corr_model_estacio)
}

mod_obs_data <- rbind(raw_model_total,
                      corr_model_total)

#Openair function that gives us the stadistics we want

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
png(file = paste0(save_general_statistics,"All_stations_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()
#-------------------------------------------------------------------------------
# Mean value for each hour all stations:
obs_total_mean          <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total_mean    <- matrix(nrow = length(date_times$date), ncol = 0)
corr_model_total_mean   <- matrix(nrow = length(date_times$date), ncol = 0)

for (i in 1:nrow(local_stations)) {
  obs_estacio        <-  df_obs_sub[ ,colnames(llista_uk_total)[i]]
  raw_model_estacio  <-  llista_uk_total_df[ ,colnames(llista_uk_total)[i]]
  corr_model_estacio <-  LOOCV_total_df[ ,colnames(llista_uk_total)[i]]

  obs_total_mean        <- cbind(obs_total_mean,obs_estacio)
  raw_model_total_mean  <- cbind(raw_model_total_mean,raw_model_estacio)
  corr_model_total_mean <- cbind(corr_model_total_mean,corr_model_estacio)
}

raw_model_mean      <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                         mod = rowMeans(raw_model_total_mean,na.rm=TRUE),
                         model="raw_model")

corr_model_mean     <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                          mod = rowMeans(corr_model_total_mean,na.rm=TRUE),
                          model="corr_model")

mod_obs_data_mean   <- rbind(raw_model_mean,
                       corr_model_mean)


Summary_stats <- openair::modStats(mod_obs_data_mean, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)

png(file = paste0(save_general_statistics,"All_stations_hourly_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()
#-------------------------------------------------------------------------------
# Per una estació:
# First, we will evalue the raw model. To do so, we need a data frame with obs and the raw model. Obv we choose df_obs_sub as obs
# colnames(llista_uk_total)[1] --> We do the stadistics for each station

#raw_model <- data.frame( obs = df_obs_sub[ ,colnames(llista_uk_total)[1] ],
                       #  mod = llista_uk_total_df[ ,colnames(llista_uk_total)[1]],
                        # model="raw_model")

#Aqui posarem el LOOC_total_df  que s'ha anat generant en el bucle amb el model corretgit
#corr_model <- data.frame( obs = df_obs_sub[ ,colnames(llista_uk_total)[1] ],
                         # mod = LOOCV_total_df[ ,colnames(llista_uk_total)[1]],
                         # model="corr_model")

#tres columnes: obs, mod i quin tipus de model es 'model': que pot ser raw_model o corr_model. Primer et posa el raw i seguidament el corr 
#mod_obs_data_1 <- rbind(raw_model,
 #                    corr_model)

# create evaluation statistics
#Summary_stats <- openair::modStats(mod_obs_data_1, obs = "obs", mod = "mod", type = "model")
#print(Summary_stats)

############################################################
#                                                          #
#                   Time series                            #
#                                                          #
############################################################
#-------------------------------------------------------------------------------
# Plot per una estació determinada, en aquest cas: colnames(llista_uk_total)[1]

# r<-2
# 
# data_single     <- data.frame(date      = date_times,
#                               observations       = df_obs_sub[ ,colnames(llista_uk_total)[r] ],
#                               Model              = llista_uk_total_df[ ,colnames(llista_uk_total)[r]], 
#                               Model_corrected    = LOOCV_total_df[ ,colnames(llista_uk_total)[r]])
# 
# 
# openair::timePlot(selectByDate(data_single, year = 2021, month = 'January'),
#                     pollutant = c("observations","Model","Model_corrected"),
#                     group = TRUE,
#                     title =colnames(llista_uk_total)[r],
#                     y.relation = "free",
#                     ylab="NO2 (ug/m3)",
#                     cols = c("black", "blue", "grey"),
#                     lty = c("dotted","solid","dashed"),
#                     lwd = c(3,2,2),
#                     name.pol = c("observations","Raw model", "Corrected Model"))

#-------------------------------------------------------------------------------
# Plot per cada estació:

month_names = c("January","February","March","April","May","June","July","August",
                 "September","October","November","December")


for (l in 1:nrow(local_stations)) {
  data_single     <- data.frame(date               = date_times,
                                observations       = df_obs_sub[ ,colnames(llista_uk_total)[l]],
                                Model              = llista_uk_total_df[ ,colnames(llista_uk_total)[l]], 
                                Model_corrected    = LOOCV_total_df[ ,colnames(llista_uk_total)[l]]) 
  #Es para al Febrer... potser no te sentit tenint només dos dies :')
  for (mm in month_names) {
    # TIME_PLOT
    png(file = paste0(save_time_series,local_stations$ref_obs[l],"_", local_stations$station_name[l],"_",pollutant,"_",mm,".png"), width = 800, height = 400)
    openair::timePlot(selectByDate(data_single, year = 2021, month = mm),
                      pollutant = c("observations","Model","Model_corrected"),
                      group = TRUE,
                      title=colnames(llista_uk_total)[l],
                      y.relation = "free",
                      ylab= paste0(pollutant, " ", "(ug/m3)"),
                      cols = c("black", "blue", "grey"),
                      lty = c("dotted","solid","dashed"),
                      lwd = c(3,2,2), 
                      name.pol = c("observations","Raw model", "Corrected Model"))
    dev.off()
}
}

#-------------------------------------------------------------------------------
# Plot per la mitjana horària de totes les estacions:

#First, we create a matrix to put the data of observations and model (corrected and raw)

month_names = c("January","February","March","April","May","June","July","August",
                "September","October","November","December")

obs_total            <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total      <- matrix(nrow = length(date_times$date), ncol = 0)
corr_model_total     <- matrix(nrow = length(date_times$date), ncol = 0)

for (i in 1:nrow(local_stations)) {
  #obs_estacio           <-  df_obs_sub[ ,colnames(llista_uk_total)[i]]
  obs_estacio           <-  df_obs_sub[ ,colnames(df_obs_sub)[i+1]]
  
  raw_model_estacio     <-  llista_uk_total_df[ ,colnames(llista_uk_total)[i]]
  corr_model_estacio    <-  LOOCV_total_df[ ,colnames(llista_uk_total)[i]]
  
  obs_total             <- cbind(obs_total,obs_estacio)
  raw_model_total       <- cbind(raw_model_total,raw_model_estacio)
  corr_model_total      <- cbind(corr_model_total,corr_model_estacio) 
  
  }
  
  #Then, we calculate the means of each hour considering the whole stations
  raw_model   <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                             mod = rowMeans(raw_model_total,na.rm=TRUE),
                             model="raw_model")
  
  corr_model   <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                              mod = rowMeans(corr_model_total,na.rm=TRUE),
                              model="corr_model")
  
  data_single   <- data.frame(date      = date_times,
                              observations       = raw_model$obs,
                              Model              = raw_model$mod,
                              Model_corrected    = corr_model$mod)

  for (mm in month_names) {
    png(file = paste0(save_general_statistics,"time_series_","hourly_mean","_",pollutant,"_",mm,".png"), width = 800, height = 400)
    openair::timePlot(selectByDate(data_single, year = 2021, month = mm),
                      pollutant = c("observations","Model","Model_corrected"),
                      group = TRUE,
                      y.relation = "free",
                      ylab= paste0(pollutant, " ", "(ug/m3)"),
                      cols = c("black", "blue", "grey"),
                      lty = c("dotted","solid","dashed"),
                      lwd = c(3,2,2),
                      name.pol = c("observations","Raw model", "Corrected Model"))
    dev.off()
    
  }
    
############################################################
#                                                          #
#              Temporal variations                         #
#                                                          #
############################################################
#-------------------------------------------------------------------------------
# Plot per la mitjana de totes les estacions:

#First, we create a matrix to put the data of observations and model (corrected and raw)
obs_total          <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total    <- matrix(nrow = length(date_times$date), ncol = 0)
corr_model_total   <- matrix(nrow = length(date_times$date), ncol = 0)

for (i in 1:nrow(local_stations)) {
  obs_estacio         <-  df_obs_sub[ ,colnames(llista_uk_total)[i]]
  raw_model_estacio   <-  llista_uk_total_df[ ,colnames(llista_uk_total)[i]]
  corr_model_estacio  <-  LOOCV_total_df[ ,colnames(llista_uk_total)[i]]
  
  obs_total        <- cbind(obs_total,obs_estacio)
  raw_model_total  <- cbind(raw_model_total,raw_model_estacio)
  corr_model_total <- cbind(corr_model_total,corr_model_estacio)
}

#Then, we calculate the means of each hour considering the whole stations
raw_model <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                         mod = rowMeans(raw_model_total,na.rm=TRUE),
                         model="raw_model")

corr_model <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                          mod = rowMeans(corr_model_total,na.rm=TRUE),
                          model="corr_model")

Mean_all_stations            <- data.frame(date            = date_times,
                                        observations       = raw_model$obs,
                                        Model              = raw_model$mod, 
                                        Model_corrected    = corr_model$mod)

TIME_VAR <- openair::timeVariation(Mean_all_stations,
                                   pollutant = c("observations","Model","Model_corrected"),
                                   ylab = paste0(pollutant, " ", "(ug/m3)"),
                                   type ="month",
                                   key = TRUE,
                                   title = "Media horaria Cat",
                                   auto.text=FALSE, 
                                   ylim = c(0, 120),
                                   cols = c("black", "blue", "grey"),
                                   lty = c("dotted","solid","dashed"),
                                   lwd = c(3,2,2), 
                                   name.pol = c("Observations", "Raw model", "Corrected Model"))  

png(file = paste0(save_general_statistics,"temporal_",pollutant,"_mean_time_VAR.png"), width = 800, height = 600)
print(TIME_VAR[[1]][2]) 
dev.off()

#-------------------------------------------------------------------------------
# Plot per cada estació:

for (n in 1:nrow(local_stations)) {
  
  Single_station            <- data.frame(date               = date_times,
                                          observations       = df_obs_sub[ ,colnames(llista_uk_total)[n]],
                                          Model              = llista_uk_total_df[ ,colnames(llista_uk_total)[n]], 
                                          Model_corrected    = LOOCV_total_df[ ,colnames(llista_uk_total)[n]])
  
  TIME_VAR <- openair::timeVariation(Single_station,
                                     pollutant = c("observations","Model","Model_corrected"),
                                     ylab = paste0(pollutant, " ", "(ug/m3)"),
                                     type ="month",
                                     key = TRUE,
                                     title = colnames(llista_uk_total)[n],
                                     auto.text=FALSE, 
                                     ylim = c(0, 120),
                                     cols = c("black", "blue", "grey"),
                                     lty = c("dotted","solid","dashed"),
                                     lwd = c(3,2,2), 
                                     name.pol = c("observations","Raw model", "Corrected Model"))  
  
  png(file = paste0(save_temporal_variation,local_stations$ref_obs[n],"_", local_stations$station_name[n],"_",pollutant,"_time_VAR.png"), width = 800, height = 600)
  print(TIME_VAR[[1]][2]) 
  dev.off()
  
}

#-------------------------------------------------------------------------------
#Temps que triga en executar-se el fitxer

end_time <- Sys.time()
temps_total <- end_time-start_time
print(temps_total)

