#Mesurem el temps que triga en executarse l'analisi d'una hora
start_time <- Sys.time()

#-------------------------------------------------------------------------------
#          UNIVERSAL KRIGING TO CALIOPE DATA WITH CATALONIA OBSERVATIONS
#-------------------------------------------------------------------------------
#HOLA

############################################################
#                                                          #
#              Set libraries                               #
#                                                          #
############################################################

# ! check the version of the package or R if any library or function does not work correctly 

library(urbanR)
library(RNetCDF) 
library(sp)
library(sf)
#library(mapview) 
library(ncdf4) #Es per el model
library(rgeos) #Es una llibreria per fer lo del buffer
library(ggplot2)
library(plyr) #OJO! la llibreria plyr ha de ser cridada abans que dplyr pk aixi ho vol el programa
library(dplyr)
library(broom)
library(magrittr)
library(ggpubr)
library(raster)
library(rgdal)
#library(tmap)
library(gstat)
library(sp)
library(abind)
#library(Metrics)
#-------------------------------------------------------------------------------

## LOAD config_file

args = commandArgs(TRUE)
if (length(args) > 0) {
  configFile_path <- args[1]
} else {
  cat("\n***WARNING: no ConfigFile was provided. Using default ConfigFile.\n")
}

#config_file it's a file where we change the data, where we want to save the results, etc. so now we extract these data from it:
#configFile_path <- "/esarchive/scratch/mhajji/RUN_UK_regional/2021_01_06/config_file.yml"
#configFile_path <- "/esarchive/scratch/mhajji/RUN_UK_regional/2021_07_12/config_file.yml"

config          <- yaml::yaml.load_file(configFile_path)
sim_folder_path <- config$sim_folder_path
obs_path        <- config$obs_path
model_path      <- config$model_path
save_result     =  config$save_result
date_time_end   <- config$date_time_end
date_time_begin <- config$date_time_begin
variogram_path  =  config$variogram_path

# As we had problems with Ordinary and Universal Kriging variograms, we saved one of each to read them in case we have problems with the one executed for a specific date or hour
variogram_ok =paste0(variogram_path, "Variogram_OK.rds")
variogram_uk =paste0(variogram_path, "Variogram_UK.rds")


############################################################
#                                                          #
#              Set temporal context                        #
#                                                          #
############################################################ 

#date_times et fa un super vector d'una  columna, dia per dia, hora per hora des de l'inici fins a la data final p.ex: 2021-01-01 00:00:00
date_times <- data_frame_dates(date_time_begin = date_time_begin,
                               date_time_end = date_time_end)

#dates: Llista de caracters anymesdia p.ex: 20210101
dates <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%m", tz = "UTC")),
                       as.character(strftime(date_times[, 1], format="%d", tz = "UTC"))))
#year_month: igual que date pero anymes p.ex: 202101
year_month <- unique(paste0(as.character(strftime(date_times[, 1], format="%Y", tz = "UTC")),
                            as.character(strftime(date_times[, 1], format="%m", tz = "UTC"))))

############################################################
#                                                          #
#                 Load Observations                        #
#                                                          #
############################################################ 


#local_stations: special points data frame with two cols [NO2 and ref_obs-reference of station-], at the moment we have 72 stations in Cat 
local_stations <- readRDS(file = paste0(sim_folder_path,"local_stations.rds")) 
#we have a doc where we have the reference of the stations that are in catalonia, if we want to change or amplify them we can change the coordinates of the box in set_stations script

pollutant      <-"no2"

#we prepare a matrix where we will save the observations:
obs            <- matrix(,ncol=nrow(local_stations@data),nrow=0)

df_obs_all =data.frame(obs)
colnames(df_obs_all)= local_stations@data$ref_obs #we transform from a matrix to a data frame, with the reference of the station as a name of the col

#finally we extract the observations of certan year_month
for (y_month in year_month) {

    netcdf_obs    <- RNetCDF::open.nc(paste0(obs_path, "/sconc", pollutant, "_", y_month, ".nc"))

    code_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'station_reference')
    obs_all       <- RNetCDF::var.get.nc(netcdf_obs, "sconcno2")

    pos_local_stations <- which(code_obs%in%local_stations@data$ref_obs)
    cat_obs<-obs_all[,pos_local_stations]
    cat_ref=code_obs[pos_local_stations]
    
    obs_month=matrix(,ncol=length(cat_obs[1,]),nrow=length(cat_obs[,1])) #cat_obs matriu de 72 col (estacions) i en aquest cas p.ex: 744 (31 dies de generx24h) i fem una obs month buida
    df_obs_month=data.frame(obs_month)
    colnames(df_obs_month) = cat_ref
    df_obs_month[,] = cat_obs

    df_obs_all              <-rbind.fill(df_obs_all,df_obs_month) #in case there is no data on a certain date, it fills it with NA (rbind.fill)
    }


############################################################
#                                                          #
#                 Load Model                               #
#                                                          #
############################################################ 

var_name_reduc       <- "sconcno2" #pollutant
n_lat                <- 278        #latitude
n_lon                <- 298        #longitude
model                <- array(dim=c(n_lat,n_lon,0)) #we prepare an array to put the date in

for (date in dates) {

   #date<-"20210101"
   conc_path_day <- paste0(date, "00")
   file <- paste(var_name_reduc, "_", conc_path_day, ".nc", sep="")
   file_path <- paste0(model_path, file)
   
   if (file.exists(file_path)) {
     netcdf_model      <- RNetCDF::open.nc(file_path) #Dintre de la llibreria executem la funció open i aplica-ho al fitxer obs

     #Extraiem les dades del model
     model_day      <- RNetCDF::var.get.nc(netcdf_model, "sconcno2") #Concentracions del NO2
     model_day      <- model_day[,,1:24]
     model_day      <- model_day*1880 #T'esta dient que les dades del model les tens en ppm i nosaltres ho volem en microg/m3, per tant x1000 i alè
   }else{ 
    model_day       <- array(NaN, dim=c(n_lat,n_lon,24))
   }
   model            <- abind(model,model_day,along=3)
}
     
    
lat_model      <- RNetCDF::var.get.nc(netcdf_model, 'lat') #Latitut
lat_model      <- sapply(lat_model, as.numeric)            #era una large matrix, ara es una large numeric
lon_model      <- RNetCDF::var.get.nc(netcdf_model, 'lon') #Longitud
lon_model      <- sapply(lon_model, as.numeric)


############################################################
#                                                          #
#                 Loop Universal Kriging                   #
#                                                          #
############################################################ 

crs_model     <- "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0" #coordinates sistem
UK_total      <-list(length=nrow(date_times)) #we will have a large list with a data.frame for each hour

for (hour in 1:nrow(date_times)) {
   print(paste0("loop UK number:",hour))
   df_model <- model[,,hour]
   
   #if all the model for a certain hour is nan we cannot apply UK
   if ( sum(is.nan(df_model))!=length(df_model)  ){
        df_obs   <- df_obs_all[hour,]

   # 1. Interpolate model values at obs locations
   #    1.1. Set spatialpointsdataframe
	df_model=data.frame(lat=lat_model,lon=lon_model,NO2=sapply(df_model,as.matrix))
        coordinates(df_model)<- ~lon+lat
	sp::proj4string(df_model) <- crs_model

        df_obs <-data.frame(lat=local_stations@coords[,2],lon=local_stations@coords[,1],NO2=as.numeric(df_obs[1,])) 
	df_obs         <- na.omit(df_obs)
	df_obs[df_obs$NO2<0,"NO2"] <- NA
	df_obs         <- na.omit(df_obs)
	coordinates(df_obs) <-~lon+lat
        sp::proj4string(df_obs) <- crs_model	

   #    1.2 Compute Variogram
        set.seed(1)
        pick_random_points <- sample(1:nrow(df_model),2000)
        NO2.fit_ok <- automap::autofitVariogram(formula=NO2~1, input_data=df_model[pick_random_points,],model="Ste") 
	# Hem posat model='Ste' pk la segona hora del 01/06/2021 peta el variograma, mirar bé perque no s'ajusa

   #    1.3 Compute ordinary Kriging

	#Del model interpolo cap a les coordenades de les obs 'OK'
	ok_gstat<- gstat(formula=NO2~1, data = df_model, model = NO2.fit_ok$var_model, nmax=100)
        #Interpolacio del model a les coordenades del conjunt
        OK <-predict(object=ok_gstat, newdata=df_obs) 
        
	#If the variogram has singular matrix then it produces NAN's, so in that case we take a good Variogram instead: 
	if(any(is.na(OK@data$var1.pred))){
        NO2.fit_ok=readRDS(variogram_ok)
	ok_gstat<- gstat(formula=NO2~1, data = df_model, model = NO2.fit_ok$var_model, nmax=100)
        #Interpolacio del model a les coordenades del conjunt
        OK <-predict(object=ok_gstat, newdata=df_obs)
        }


   # Arrange data obs-model
       llista_uk               <- data.frame(df_obs@coords)
       names(llista_uk)        = c("lon", "lat") #crec que no fa falta
       llista_uk$NO2_obs       <-df_obs@data$NO2
       llista_uk$NO2_model     <- OK$var1.pred
       coordinates(llista_uk)  <- ~lon+lat
       sp::proj4string(llista_uk) <- crs_model
       
       # Compute UK variogram

       #in exceptional cases we have problems with autofitVariogram (eg: date="20211108" at 18 h) we still don't know why (maybe few points to execute the variogram, also we see negative range...)
       # So, in tryCatch puts a NaN in NO2.fit_uk if autofitVariogram don't work

       NO2.fit_uk <- automap::autofitVariogram(formula=NO2_obs~NO2_model,input_data=llista_uk, verbose=TRUE)
       
       if(any(NO2.fit_uk$var_model$range<0)){
	       NO2.fit_uk=readRDS(variogram_uk)      
        }

   # UK
       uk_gstat<- gstat(formula=NO2_obs~NO2_model,data=llista_uk, model = NO2.fit_uk$var_model)
       names(df_model) <- c('NO2_model')
       UK <-predict(object=uk_gstat, newdata=df_model)
    
   # Save UK

   UK_total[[hour]] <- UK@data$var1.pred
   }else{
   UK_total[[hour]] <- array(NA, dim=length(df_model))
   }
}

#saveRDS save data into R Data Format, we save the UK results in one doc, and the coordinares in another for memory reasons:

saveRDS(object=UK_total, file=paste0(save_result,"/Uk_results.rds"))
saveRDS(object=UK, file=paste0(save_result,"/Uk_spatialpoints.rds"))



end_time <- Sys.time()

temps_total <- end_time-start_time
print(temps_total)



