# Measure the time it takes to execute the file
start_time_stations <- Sys.time()

############################################################
#                                                          #
#                      Set stations                        #
#                                                          #
############################################################
# study area limitation:
my_box          <-
  rgeos::bbox2SP(43.02355,
                 40.24870,
                 3.497162,
                 -0.001190186,
                 proj4string = CRS(crs_latlon)) # lat màx, lat min, lon max, lon min

if (file_format == "ghost") {
  all_stations = NULL
  for (i in 1:length(year_month)) {
    obs_path_stations <- paste0(obs_path, name_pollutant, "_", year_month[i], ".nc")
    
    #Dades de les estacions:
    netcdf_obs        <- RNetCDF::open.nc(obs_path_stations)
    
    #Arreglem les dades de les observacions
    lat_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'latitude')
    lon_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'longitude')
    ref_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'station_reference')
    station_name <- RNetCDF::var.get.nc(netcdf_obs, 'station_name')
    obs_all      <- RNetCDF::var.get.nc(netcdf_obs, name_pollutant) # no la fem servir, eliminar?
    
    df_obs       <-
      data.frame(
        pollutant = NA,
        lat_obs = lat_obs,
        lon_obs = lon_obs,
        ref_obs = ref_obs,
        station_name = station_name
      )
    all_stations <- rbind(all_stations, df_obs)
  }
  
  df2               <- all_stations[!duplicated(all_stations),]
  coordinates(df2)  <- ~ lon_obs + lat_obs
  
  #Coordenades
  sp::proj4string(df2)   <- crs_latlon

  #Extraiem les coordenades de les observacions que es troben dintre de l'àrea limitada 'my box'
  conjunt         <- raster::intersect(df2, my_box) #Fa una llista de les obs de NO2 que estiguin dindre de l'àrea del box
  
  local_stations  <- conjunt
  
  
} else if (file_format == "gene") {
  all_stations = NULL
  for (n in 1:length(read_year)) {
    ############################################################
    #           !!!!   WARNING!!!!                             #
    ############################################################
    #The stations are refered to 2020 data, it need to be revised
    df_local_stations           <- read.csv(paste0(stations_path,"XVPCA_info_",name_pollutant,"_","2020",".csv")) #ojo con el read_year, millor: read_year[n]
    df_local_stations$pollutant <- NA
    df_local_stations <- df_local_stations %>%
      rename(ref_obs = code)
    df_local_stations$ref_obs   <- as.character(df_local_stations$ref_obs)
    df_local_stations$type      <- as.character(df_local_stations$type)
    
    # lat_obs      <- df_local_stations$lat
    # lon_obs      <- df_local_stations$lon
    # ref_obs      <- df_local_stations$code
    # station_type <- df_local_stations$type
    
    # df_obs       <-
    #   data.frame(
    #     pollutant = NA,
    #     lat_obs = lat_obs,
    #     lon_obs = lon_obs,
    #     ref_obs = ref_obs,
    #     station_type = station_type
    #   )
    # all_stations <- rbind(all_stations, df_obs)
    
    
    all_stations       <- rbind(all_stations, df_local_stations)
  
    }
  
  df2                   <- all_stations[!duplicated(all_stations),]
  coordinates(df2)      <- ~ lon + lat
  sp::proj4string(df2)  <- crs_latlon
  
  # Selecting local stations within a certain area (not needed at the moment)
  #Extraiem les coordenades de les observacions que es troben dintre de l'àrea limitada 'my box'
  conjunt         <- raster::intersect(df2, my_box) #Fa una llista de les obs de NO2 que estiguin dindre de l'àrea del box
  local_stations  <- conjunt
  
  
} else {
  print("wrong obs_dataset")
}

# m=mapview(local_stations_gene, col.regions="#BEBEBE80") + local_stations_ghost
# mapshot(m, url = "/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/local_stations/comparison_stations.html")

#saveRDS save data into R Data Format
if (save_stations) {
  saveRDS(object = local_stations,file = paste0(save_result,"local_stations_",file_format,"_",pollutant,".rds"))
  print("Local stations saved")
} else {
  print("Local stations have not been saved")
}


end_time_stations    <- Sys.time()
temps_total_stations <- end_time_stations - start_time_stations
print(paste0("local stations, execution time:",temps_total_stations, "s" ))
