############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################

# it alerts us if the dates in model do not coincide with the dates of the obs
if ( any(date_times$date != df_obs_sub$date))  {
  print("Warning: dates from model and observations are different")
}

#-------------------------------------------------------------------------------
#We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Comparation using all stations concatenated:
raw_model_total         <- NULL
UK_model_total          <- NULL 
Burnes_model_total      <- NULL 
geostats_results        <- NULL

for (station_code in local_stations@data$ref_obs) {
  # station_code <- local_stations@data$ref_obs[1]
  if (station_code %in% colnames(df_obs_sub)) {
    obs_station <- df_obs_sub[ ,station_code]
  } else {
      obs_station <- NA
      }
  raw_model_estacio <- data.frame(obs = obs_station,
                                  mod = model_raw_at_AQMS_df[ ,station_code], #Jan: Canviar nom model_raw_at_AQMS_df -> raw_model_stations aqui o tots els scripts
                                  model="raw_model")
  UK_model_estacio <-   data.frame(obs = obs_station,
                                   mod = LOOCV_total_df[ ,station_code], # sugg LOOCV_total_df -> UK_loocv ("idem"?)
                                   model="UK_model")
  # Read burns data
  label <- paste0(burnes_path,"Cross_Validation_",station_code,".csv")
    if (file.exists(label)) {
    df <- read.csv(label, header = TRUE)
    burnes_loocv <-df[,c(2)]
  } else{
    burnes_loocv <- NA
  }
  Burnes_model_estacio <-   data.frame(obs = obs_station,
                                       mod = burnes_loocv,
                                       model="burnes_model")
  
  # Statistics for a single station
  Raw_geostats    <- openair::modStats(raw_model_estacio, obs = "obs", mod = "mod", type = "model")
  UK_geostats     <- openair::modStats(UK_model_estacio, obs = "obs", mod = "mod", type = "model")
  Burnes_geostats <- openair::modStats(Burnes_model_estacio, obs = "obs", mod = "mod", type = "model")
  
  geostats_station <- data.frame(x=local_stations@coords[which(local_stations@data$ref_obs == station_code),1],
                                 y=local_stations@coords[which(local_stations@data$ref_obs == station_code),2],
                                 COE_raw = Raw_geostats$COE,
                                 COE_UK = UK_geostats$COE,
                                 COE_Burnes = Burnes_geostats$MB,
                                 MB_raw = Raw_geostats$MB,
                                 MB_UK = UK_geostats$MB,
                                 MB_Burnes = Burnes_geostats$MB)
  
  # Gather all stations
  geostats_results    <- rbind(geostats_results,geostats_station)
  #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
  raw_model_total     <- rbind(raw_model_total,raw_model_estacio) 
  UK_model_total      <- rbind(UK_model_total,UK_model_estacio)
  Burnes_model_total  <- rbind(Burnes_model_total,Burnes_model_estacio)
}

mod_obs_data <- rbind(raw_model_total,
                      UK_model_total,
                      Burnes_model_total)

#Openair function that gives us the stadistics we want

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
png(file = paste0(save_general_statistics,"All_stations_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()

print("Statistic for all stations concateneited: ")

##
## Map Statistics
##



geostats_results <- na.omit(geostats_results)

coordinates(geostats_results)        <- ~ x + y
sp::proj4string(geostats_results)    <- crs_latlon

mapview::mapview(geostats_results, zcol = "COE_UK", legend = TRUE,at=c(-1,-0.5,0,0.5,1))
mapview::mapview(geostats_results, zcol = "COE_Burnes", legend = TRUE,at=c(-1,-0.5,0,0.5,1))

mapview::mapview(geostats_results, zcol = "MB_UK", legend = TRUE,at=c(-10,-5,0,5,10))
mapview::mapview(geostats_results, zcol = "MB_Burnes", legend = TRUE,at=c(-10,-5,0,5,10))

# #config the coordinates in UTM
# if (compute_in_UTM) {
#   # Jan: not working! sp::proj4string(llista_loocv) <- crs_utm
#   llista_loocv <- spTransform(llista_loocv, sp::CRS(crs_utm))
# } else {
#   sp::proj4string(llista_loocv) <- crs_latlon
# }





#-------------------------------------------------------------------------------
# Mean value for each hour all stations:
obs_total_mean          <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total_mean    <- matrix(nrow = length(date_times$date), ncol = 0)
UK_model_total_mean   <- matrix(nrow = length(date_times$date), ncol = 0)
Burnes_model_total_mean   <- matrix(nrow = length(date_times$date), ncol = 0)

for (station_code in local_stations@data$code) {
  
  # station_code <- local_stations@data$code[1]
  if (station_code %in% colnames(df_obs_sub)) {obs_station <- df_obs_sub[ ,station_code]}
  else {obs_station <- NA}
  
  obs_estacio        <-  obs_station
  raw_model_estacio  <-  model_raw_at_AQMS_df[ ,station_code]
  UK_model_estacio   <-  LOOCV_total_df[ ,station_code]
  
  # Read burns data
  label <- paste0(burnes_path,"Cross_Validation_",station_code,".csv")
  if (file.exists(label)) {
    df <- read.csv(label, header = TRUE)
    burnes_loocv <-df[,c(2)]
  } else{
    burnes_loocv <- NA
  }
  
  obs_total_mean        <- cbind(obs_total_mean,obs_estacio)
  raw_model_total_mean  <- cbind(raw_model_total_mean,raw_model_estacio)
  UK_model_total_mean <- cbind(UK_model_total_mean,UK_model_estacio)
  Burnes_model_total_mean <- cbind(Burnes_model_total_mean,burnes_loocv)
}

raw_model_mean      <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(raw_model_total_mean,na.rm=TRUE),
                                   model="raw_model")

UK_model_mean     <- data.frame(   obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(UK_model_total_mean,na.rm=TRUE),
                                   model="UK_model")

Burnes_model_mean     <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                     mod = rowMeans(Burnes_model_total_mean,na.rm=TRUE),
                                     model="Burnes_model")

mod_obs_data_mean   <- rbind(raw_model_mean,
                             UK_model_mean,
                             Burnes_model_mean)


Summary_stats <- openair::modStats(mod_obs_data_mean, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)

png(file = paste0(save_general_statistics,"All_stations_hourly_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()

print("Statistic considering the mean hour for all stations: ")

#-------------------------------------------------------------------------------
# Per una estació:
# First, we will evalue the raw model. To do so, we need a data frame with obs and the raw model. Obv we choose df_obs_sub as obs
# colnames(model_raw_at_AQMS)[1] --> We do the stadistics for each station
# 
# raw_model <- data.frame( obs = df_obs_sub[ ,colnames(model_raw_at_AQMS)[1] ],
#                          mod = model_raw_at_AQMS_df[ ,colnames(model_raw_at_AQMS)[1]],
#                          model="raw_model")
# 
# #Aqui posarem el LOOC_total_df  que s'ha anat generant en el bucle amb el model corretgit
# corr_model <- data.frame( obs = df_obs_sub[ ,colnames(model_raw_at_AQMS)[1] ],
#                           mod = LOOCV_total_df[ ,colnames(model_raw_at_AQMS)[1]],
#                           model="corr_model")
# 
# #tres columnes: obs, mod i quin tipus de model es 'model': que pot ser raw_model o corr_model. Primer et posa el raw i seguidament el corr
# mod_obs_data_1 <- rbind(raw_model,
#                       corr_model)
# 
# # create evaluation statistics
# print("Example of the statistics for one station: ")
# Summary_stats <- openair::modStats(mod_obs_data_1, obs = "obs", mod = "mod", type = "model")
# print(Summary_stats)


