############################################################
#                                                          #
#              Temporal variations                         #
#                                                          #
############################################################
#-------------------------------------------------------------------------------
# Plot per la mitjana de totes les estacions:

#First, we create a matrix to put the data of observations and model (corrected and raw)
obs_total          <- matrix(nrow = length(date_times$date), ncol = 0) 
raw_model_total    <- matrix(nrow = length(date_times$date), ncol = 0)
UK_model_total     <- matrix(nrow = length(date_times$date), ncol = 0)
Burnes_model_total <- matrix(nrow = length(date_times$date), ncol = 0)


for (station_code in local_stations@data$code) {
  
  if (station_code %in% colnames(df_obs_sub)) {obs_station <- df_obs_sub[ ,station_code]}
  else {obs_station <- NA}  
  obs_estacio         <-  obs_station
  raw_model_estacio   <-  model_raw_at_AQMS_df[ ,station_code]
  UK_model_estacio    <-  LOOCV_total_df[ ,station_code]

  # Read burns data
  label <- paste0(burnes_path,"Cross_Validation_",station_code,".csv")
  if (file.exists(label)) {
    df <- read.csv(label, header = TRUE)
    burnes_loocv <-df[,c(2)]
  } else{
    burnes_loocv <- NA
  }
  
    
  obs_total        <- cbind(obs_total,obs_estacio)
  raw_model_total  <- cbind(raw_model_total,raw_model_estacio)
  UK_model_total <- cbind(UK_model_total,UK_model_estacio)
  Burnes_model_total <- cbind(Burnes_model_total,burnes_loocv)
}

#Then, we calculate the means of each hour considering the whole stations
raw_model <- data.frame( obs = rowMeans(obs_total,na.rm=TRUE),
                         mod = rowMeans(raw_model_total,na.rm=TRUE),
                         model="raw_model")

UK_model <- data.frame(   obs = rowMeans(obs_total,na.rm=TRUE),
                          mod = rowMeans(UK_model_total,na.rm=TRUE),
                          model="corr_model")

Burnes_model <- data.frame(   obs = rowMeans(obs_total,na.rm=TRUE),
                          mod = rowMeans(Burnes_model_total,na.rm=TRUE),
                          model="corr_model")


Mean_all_stations            <- data.frame(date            = date_times,
                                           observations       = raw_model$obs,
                                           raw_Model          = raw_model$mod, 
                                           UK_Model           = UK_model$mod,
                                           Burnes_Model       = Burnes_model$mod)

TIME_VAR <- openair::timeVariation(Mean_all_stations,
                                   pollutant = c("observations","raw_Model","UK_Model","Burnes_Model"),
                                   ylab = paste0(pollutant, " ", "(ug/m3)"),
                                   type ="month",
                                   key = TRUE,
                                   title = "Media horaria Cat",
                                   auto.text=FALSE,
                                   ci = FALSE,
                                   conf.int = c(0.75, 0.99),
                                   ylim = c(0, 35),
                                   cols = c("black", "grey", "green","brown"),
                                   lty = c("solid","solid","solid","solid"),
                                   lwd = c(8,2,2,2), 
                                   name.pol = c("Observations", "Raw model", "UK Model","Burnes Model"))  

png(file = paste0(save_general_statistics,"temporal_",pollutant,"_mean_time_VAR.png"), width = 800, height = 600)
print(TIME_VAR[[1]][2]) 
dev.off()
#-------------------------------------------------------------------------------
# Plot per cada estació:


for (station_code in local_stations@data$code) {
  
  # station_code <- local_stations@data$code[1]
  if (station_code %in% colnames(df_obs_sub)) {obs_station <- df_obs_sub[ ,station_code]
  } else {obs_station <- NA}  
  obs_estacio         <-  obs_station
  raw_model_estacio   <-  model_raw_at_AQMS_df[ ,station_code]
  UK_model_estacio    <-  LOOCV_total_df[ ,station_code]
  
  # Read burns data
  label <- paste0(burnes_path,"Cross_Validation_",station_code,".csv")
  if (file.exists(label)) {
    df <- read.csv(label, header = TRUE)
    burnes_loocv <-df[,c(2)]
  } else{
    burnes_loocv <- NA
  }
  #ref_names <- 
  
  Single_station            <- data.frame(date               = date_times,
                                          observations       = obs_estacio,
                                          raw_Model          = raw_model_estacio, 
                                          UK_corrected       = UK_model_estacio,
                                          Burnes_corrected   = burnes_loocv)
  
  TIME_VAR <- openair::timeVariation(Single_station,
                                     pollutant = c("observations","raw_Model","UK_corrected","Burnes_corrected"),
                                     ylab = paste0(pollutant, " ", "(ug/m3)"),
                                     type ="month",
                                     key = TRUE,
                                     title = station_code,
                                     auto.text=FALSE, 
                                     ci = FALSE,
                                     conf.int = c(0.75, 0.99),
                                     ylim = c(0, 100),
                                     cols = c("black", "grey", "green","brown"),
                                     lty = c("solid","solid","solid","solid"),
                                     lwd = c(8,2,2,2), 
                                     name.pol = c("Observations", "Raw model", "UK Model","Burnes Model"))  
  png(file = paste0(save_temporal_variation,station_code,"_",pollutant,"_time_VAR.png"), width = 800, height = 600)
  print(TIME_VAR[[1]][2]) 
  dev.off()
  
}

